bool send_memory_buffer(Net::Socket@ sock, MemoryBuffer@ buf)
{
	if (!sock.Write(buf))
	{
		// If this fails, the socket might not be open. Something is wrong!
		print("INFO: Disconnected, could not send data.");
		return false;
	}
	return true;
}

// cast val to a float when necessary and append it to buf:
// "As the scripting engine has been optimized for 32 bit datatypes, using the smaller variants is only recommended for accessing application specified variables. For local variables it is better to use the 32 bit variant."
void append_float(MemoryBuffer@ buf, float val)
{
	buf.Write(val);
}

void append_bool(MemoryBuffer@ buf, bool val)
{
	if (val)
	{
		buf.Write(1.0f);
	}
	else
	{
		buf.Write(0.0f);

	}
}

void append_int(MemoryBuffer@ buf, int32 val)
{
	buf.Write(float(val));
}

void Main()
{

	float prev_speed = 0;
    float speed = 0;
	float prev_acceleration = 0;
    float acceleration = 0;
	float jerk = 0;
	bool isBraking = false;
	bool isFinished = false;
    int _curCP = 0;
    int _curLap = 0;
	while(true)
	{
		CSceneVehicleVisState@ vehicle = VehicleState::ViewingPlayerState();

		CHmsCamera@ cam = Camera::GetCurrent();

		auto sock_serv = Net::Socket();
		if (!sock_serv.Listen("127.0.0.1", 9000)) {
			print("Could not initiate server socket.");
			return;
		}
		print(Time::Now + ": Waiting for incoming connection...");

		while(!sock_serv.CanRead()){
			yield();
		}
		print("Socket can read");
		auto sock = sock_serv.Accept();

		print(Time::Now + ": Accepted incomming connection.");

		while (!sock.CanWrite()) {
			yield();
		}
		print("Socket can write");
		print(Time::Now + ": Connected!");

		// OpenPlanet can store bytes in a MemoryBuffer:
		MemoryBuffer@ buf = MemoryBuffer(0);

		bool cc = true;
		while(cc)
		{
			CTrackMania@ app = cast<CTrackMania>(GetApp());
			if(app is null)
			{
				yield();
				continue;
			}
			CSmArenaClient@ playground = cast<CSmArenaClient>(app.CurrentPlayground);
			if(playground is null)
			{
				yield();
				continue;
			}
			CSmArena@ arena = cast<CSmArena>(playground.Arena);
			if(arena is null)
			{
				yield();
				continue;
			}
			if(arena.Players.Length <= 0)
			{
				yield();
				continue;
			}

			auto player = arena.Players[0];
			if(player is null)
			{
				yield();
				continue;
			}

			CSmScriptPlayer@ api = cast<CSmScriptPlayer>(player.ScriptAPI);
			if(api is null)
			{
				yield();
				continue;
			}
			if(vehicle is null)
			{
				yield();
				continue;
			}
			auto race_state = playground.GameTerminals[0].UISequence_Current;

			speed = api.Speed;
      		acceleration = speed - prev_speed;
			jerk = acceleration - prev_acceleration;
      		prev_speed = speed;
			prev_acceleration = acceleration;
			isBraking = api.InputIsBraking;

            if(race_state == SGamePlaygroundUIConfig::EUISequence::Finish || race_state == SGamePlaygroundUIConfig::EUISequence::EndRound)
			{
				isFinished = true;
			}
			else
			{
				isFinished = false;
			}

			if(PlayerState::GetRaceData().PlayerState == PlayerState::EPlayerState::EPlayerState_Driving) 
			{
				auto info = PlayerState::GetRaceData().dPlayerInfo;
				_curCP = info.NumberOfCheckpointsPassed;
				_curLap = info.CurrentLapNumber;
			} 
			else 
			{
				_curCP = 0;
				_curLap = 0;
			}

			buf.Seek(0, 0);
			// Sending data
            append_float(buf, _curCP); // 0
            append_float(buf, _curLap);	

			append_float(buf, vehicle.FrontSpeed); // 2

			append_float(buf, api.Position.x); 
			append_float(buf, api.Position.y); // 4
			append_float(buf, api.Position.z);
			append_float(buf, api.InputSteer); // 6
			append_float(buf, api.InputGasPedal);
			append_bool(buf, isBraking); // 8

			append_bool(buf, isFinished);

			append_float(buf, acceleration); // 10
			append_float(buf, jerk);
			
      		append_float(buf, api.AimYaw); // 12
			append_float(buf, api.AimPitch);

			append_float(buf, vehicle.FLSteerAngle);
			append_float(buf, vehicle.FRSteerAngle); // 15

            append_float(buf, vehicle.FLSlipCoef); 
			append_float(buf, vehicle.FRSlipCoef); // 17	
			
			append_float(buf, api.EngineCurGear); // 18 int


			buf.Seek(0, 0);
			// 19 danych
	    	cc = send_memory_buffer(sock, buf);

			yield();  // this statement stops the script until the next frame
		}
		sock.Close();
		sock_serv.Close();
	}
}